@@ -1,14 +1,11 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from decimal import Decimal
-from trytond.model import fields
-from trytond.pyson import Eval
+from trytond.model import fields, Check
 from trytond.pool import Pool, PoolMeta
 from trytond.transaction import Transaction
-from trytond import backend
 
 __all__ = ['Move']
-__metaclass__ = PoolMeta
 
 
 def _get_field(type_):
@@ -18,45 +15,29 @@ def _get_field(type_):
         return 'out_anglo_saxon_quantity'
 
 
-class Move:
+class Move(metaclass=PoolMeta):
     __name__ = 'stock.move'
     in_anglo_saxon_quantity = fields.Float('Input Anglo-Saxon Quantity',
-        required=True, digits=(16, Eval('unit_digits', 2)),
-        depends=['unit_digits'])
+        required=True)
     out_anglo_saxon_quantity = fields.Float('Output Anglo-Saxon Quantity',
-        required=True, digits=(16, Eval('unit_digits', 2)),
-        depends=['unit_digits'])
+        required=True)
 
     @classmethod
     def __setup__(cls):
         super(Move, cls).__setup__()
         cls._allow_modify_closed_period.update(['in_anglo_saxon_quantity',
                 'out_anglo_saxon_quantity'])
+
+        t = cls.__table__()
         cls._sql_constraints += [
             ('check_in_anglo_saxon_quantity',
-                'CHECK(quantity >= in_anglo_saxon_quantity)',
+                Check(t, t.quantity >= t.in_anglo_saxon_quantity),
                 'Anglo-Saxon quantity can not be greater than quantity.'),
             ('check_out_anglo_saxon_quantity',
-                'CHECK(quantity >= out_anglo_saxon_quantity)',
+                Check(t, t.quantity >= t.out_anglo_saxon_quantity),
                 'Anglo-Saxon quantity can not be greater than quantity.'),
             ]
 
-    @classmethod
-    def __register__(cls, module_name):
-        cursor = Transaction().cursor
-        TableHandler = backend.get('TableHandler')
-
-        super(Move, cls).__register__(module_name)
-        table = TableHandler(cursor, cls, module_name)
-
-        # Migration from 2.8: split anglo_saxon_quantity
-        if table.column_exist('anglo_saxon_quantity'):
-            cursor.execute('UPDATE "' + cls._table + '" '
-                'SET in_anglo_saxon_quantity = anglo_saxon_quantity, '
-                'out_anglo_saxon_quantity = anglo_saxon_quantity')
-            table.drop_constraint('check_anglo_saxon_quantity')
-            table.drop_column('anglo_saxon_quantity')
-
     @staticmethod
     def default_in_anglo_saxon_quantity():
         return 0.0
@@ -153,8 +134,7 @@ class Move:
         pool = Pool()
         Uom = pool.get('product.uom')
 
-        for move in moves:
-            assert move.product == product, 'wrong product'
+        assert all(m.product == product for m in moves), 'wrong product'
         assert type_.startswith('in_') or type_.startswith('out_'), \
             'wrong type'
 
@@ -170,12 +150,16 @@ class Move:
 
             cost += move_cost_price * Decimal(str(move_qty))
 
-            move_qty = Uom.compute_qty(product.default_uom, move_qty, move.uom)
-            with Transaction().set_user(0, set_context=True):
-                cls.write([move], {
-                        as_qty_field: (
-                            (getattr(move, as_qty_field) or 0.0) + move_qty),
-                        })
+            move_qty = Uom.compute_qty(
+                product.default_uom, move_qty, move.uom, round=False)
+
+            # Avoid float rounding issue but allow only rounding precision lost
+            new_qty = (getattr(move, as_qty_field) or 0.0) + move_qty
+            assert move.uom.round(new_qty) <= move.quantity
+            new_qty = min(new_qty, move.quantity)
+            cls.write([move], {
+                    as_qty_field: new_qty,
+                    })
 
         if consumed_qty < total_qty:
             qty = total_qty - consumed_qty
@@ -187,7 +171,8 @@ class Move:
     def copy(cls, moves, default=None):
         if default is None:
             default = {}
-        default = default.copy()
+        else:
+            default = default.copy()
         for prefix in ('in_', 'out_'):
             default.setdefault(prefix + 'anglo_saxon_quantity',
                 getattr(cls, 'default_%sanglo_saxon_quantity' % prefix)())
